classdef domain
    properties
        headGrid
        membershipCube % boolean 3D array which isd true on grid points that belong to the domain
        exemplarLinearMeasure
        exemplarLocationId
        linearizedProjectedMeasure
        label
        dipoleAndMeasure % contain a copy of the original dipoleAndMeasure but with the most memory-consuming part (linearizedMeasure field) removed.
        scalpMapVisualizationPolarity = 1; % used primarily for making ERP projection to scalp positive.
        standardDeviationOfEstimatedDipoleLocation; % the value which was used to create the domain
    end % properties
    methods
        function obj = domain(membershipCube, headGrid, linearizedProjectedMeasure, exemplarLocationId, exemplarLinearMeasure, label, dipoleAndMeasure, standardDeviationOfEstimatedDipoleLocation)
            % linearizedProjectedMeasure is an N x P matrix, which N is the dimension of the
            % linearized measure and P is the number members in the domain.
            if nargin>0
                obj.membershipCube = membershipCube;
                obj.headGrid = headGrid;
                obj.linearizedProjectedMeasure = linearizedProjectedMeasure;
                obj.exemplarLocationId = exemplarLocationId;
                obj.exemplarLinearMeasure = exemplarLinearMeasure;
                obj.dipoleAndMeasure = dipoleAndMeasure;
                obj.standardDeviationOfEstimatedDipoleLocation = standardDeviationOfEstimatedDipoleLocation;
                
                if nargin > 5
                    obj.label = label;
                end;
            end;
        end;
        function valueOnFineGrid = plotMri(obj, mri3dplotOptions)
            % to do: add an option to color the plot by correlation ,intead of solid lines
            
            if nargin<2
                mri3dplotOptions = {'mriview' , 'top','mrislices', [-50 -30 -20 -15 -10 -5 0 5 10 15 20 25 30 40 50]};
            end;
            
            valueOnCoarseGrid = double(obj.membershipCube);
            
            valueOnCoarseGrid(valueOnCoarseGrid<0) = 0;
            
            [valueOnFineGrid mri] = convert_coarse_grid_to_mri3d(valueOnCoarseGrid, obj.headGrid.xCube, obj.headGrid.yCube, obj.headGrid.zCube);
            mri3dplot(valueOnFineGrid, mri, mri3dplotOptions{:}); % for some reason, this function hicjacks a currently open figure. even if a new figur is just created.
            title([obj.label ' membership'], 'color', [1 1 1]);
        end;
        function plotScatter(obj)
            figure;
            
            t = double(obj.membershipCube);
            t(~obj.membershipCube) = eps;
            
            color = value2color(t(:), jet);
            
            scatter3(obj.headGrid.xCube(obj.headGrid.insideBrainCube), obj.headGrid.yCube(obj.headGrid.insideBrainCube), obj.headGrid.zCube(obj.headGrid.insideBrainCube), (t(obj.headGrid.insideBrainCube) * 40)+5, color(obj.headGrid.insideBrainCube), 'filled');
            axis equal;
        end
        function plotMeasure(obj) % plots both the measure and the location
            
            % for erp (of study), use erpVisualizationPolarity so erp polarity can be inverted, just
            % for visualization.
            if strcmp(class(obj.dipoleAndMeasure), 'pr.dipoleAndMeasureOfStudyErp')
                obj.dipoleAndMeasure.plot(obj.scalpMapVisualizationPolarity * obj.exemplarLinearMeasure);
            else
                obj.dipoleAndMeasure.plot(obj.exemplarLinearMeasure);
            end;
        end;               
        function plotConditionDifference(obj, dipoleAndMeasure, varargin)
            % plotConditionDifference(obj, dipoleAndMeasure, {optional 'key', 'value' inputs})
            %
            % plot condition difference between first and second specified conditions (A-B) and
            % mask areas with non-significent differences.
            %
            % Optional 'key', 'value' inputs:
            %
            %   'significanceLevel'       - p-value number for masking condition difference. {default: 0.03}
            %   'twoConditionLabels'      - a cell containing the labels of the two labels to be
            %                               compared. By default the first two are compared.
            %   'statisticsParameter'     - a cell containing statistical parameters for comapring 
            %                               two conditions. These parameters are passed to
            %                               statcond() function. See statcond(0 for more detils.
            %                               Example: {'mode', 'perm', 'naccu', 250} to perform permutation
            %                               statistics with 250 random permutations.
            %   'plottingParameter'       - a cell containing additional plotting parameters to be
            %                               pased to the plotting function appropriate for that measure, like std_plottf() for
            %                               ERSP or std_plotcurve() for ERP.
            %   'positionsForStatistics' -  a string that selects the method with which the
            %                               projected measures on domain positions are used in
            %                               statistical comparison between the conditions:
            %                               'each': uses projection from each point of the domain as a sepatrate
            %                               and equivalent measurement for that session.
            %                               'mean': calculate the mean projected measure over the
            %                               domain location.
            %                               'exemplar': only uses th projection and exemplar
            %                               location.
            %                               {default: 'each' which seems to produce best results, as
            %                               more measurements for each session are provided}
            %   'positionForMeasure'     -  position for which the projected measure difference is calculated, 
            %                               masked and then plotted. Masking is always controlled by
            %                               'positionsForStatistics' parameters, but
            %                               'positionForMeasure' parameters specifies the projected measure for 
            %                                which the difference between the two conditions is
            %                                plotted.
            %                                'exemplar': use projected measure at exemplar location.
            %                                'mean;: use mean projected measure over all domain members (locations).                
            %                                                                                   
            
            if nargin < 2
                error('Please provide a variable of type dipoleAndMeasure in the first argument.');
            end;
             
            inputOptions = finputcheck(varargin, ...
                { 'significanceLevel'     'real'     []                        0.03; ...
                'twoConditionLabels'      'cell'     {}                         {};...
                'statisticsParameter'     'cell'     {}                         {};...
                'plottingParameter'       'cell'     {}                         {};...
                'positionsForStatistics'  'string' {'each' 'mean' 'exemplar'}   'each';...
                'positionForMeasure'      'string' {'exemplar' 'mean'}          'exemplar';...
                });
            
            if strcmpi(inputOptions.positionsForStatistics, 'exemplar')
                positionIdForStatistics = obj.exemplarLocationId;
            else
                positionIdForStatistics = find(obj.membershipCube); % default, provide all positions in the domain for statistics. 
            end;
            
            if strcmpi(inputOptions.positionForMeasure, 'exemplar')
                % for condition comparison, it is better to compare projections to the exemplar
                % location.
                dipoleAndMeasure.plotConditionDifference(obj.exemplarLinearMeasure, obj.headGrid.getPosition(positionIdForStatistics), obj.standardDeviationOfEstimatedDipoleLocation, inputOptions.twoConditionLabels, inputOptions.significanceLevel, inputOptions.positionsForStatistics, inputOptions.statisticsParameter, inputOptions.plottingParameter);                
            else % use average measure over the whole domain when inputOptions = 'mean'
                meanDomainLinearizedProjectedMeasure = mean(obj.linearizedProjectedMeasure,2);
                dipoleAndMeasure.plotConditionDifference(meanDomainLinearizedProjectedMeasure, obj.headGrid.getPosition(positionIdForStatistics), obj.standardDeviationOfEstimatedDipoleLocation, inputOptions.twoConditionLabels, inputOptions.significanceLevel,inputOptions.positionsForStatistics,  inputOptions.statisticsParameter, inputOptions.plottingParameter);
            end;
            
        end;
        function plotGroupDifference(obj, dipoleAndMeasure, varargin)
            % plotGroupDifference(obj, dipoleAndMeasure, significanceLevelForConditionDifference, twoGroupLabelsForComparison, conditionLabelsForComparison, statisticsParameter, plottingParametervarargin)
            %
            % plot group difference between first and second specified group (A-B) for a condition and
            % mask areas with non-significent differences.
            
            inputOptions = finputcheck(varargin, ...
                { 'significanceLevel'     'real'     []                        0.03; ...
                'twoConditionLabels'      'cell'     {}                         {};...
                'conditionLabel'         'string'    {}                         {};...
                'statisticsParameter'     'cell'     {}                         {};...
                'plottingParameter'       'cell'     {}                         {};...
                'positionsForStatistics'  'string' {'each' 'mean' 'exemplar'}   'each';...
                'positionForMeasure'      'string' {'exemplar' 'mean'}          'exemplar';...
                });
            
            if nargin < 2
                error('Please provide a variable of type dipoleAndMeasure in the first argument.');
            end;
            
            if nargin < 3
                significanceLevelForConditionDifference = 0.03;
            end;
            
            % if no group label is provided, use the default which is the first two groups.
            if nargin < 4
                twoGroupLabelsForComparison = {};
            end;
            
             % if no condition label is provided, use the default which is the first condition.
            if nargin < 5
                conditionLabelForComparison = {};
            end;
            
            if nargin < 6
                statisticsParameter = {};
            end;
            
            if nargin < 7
                plottingParameter = {};
            end;
                     
            % for group comparison, we compare average projection to all domain locations.
            dipoleAndMeasure.plotGroupDifference(obj.exemplarLinearMeasure, obj.headGrid.getPosition(obj.membershipCube), obj.standardDeviationOfEstimatedDipoleLocation, twoGroupLabelsForComparison, conditionLabelForComparison, significanceLevelForConditionDifference, statisticsParameter, plottingParameter);
        end;
        function plotVolume(obj, regionColor, regionOptions, createNewFigure)
            
            if nargin<2
                regionColor = 'g';
            end;
            
            if nargin<3
                regionOptions = {};
            end;
            
            if nargin<4
                createNewFigure = true;
            end;
            
            if createNewFigure
                figure;
            end;
            
            plot_dipplot_with_cortex;
            plot_head_region(obj.headGrid, obj.membershipCube, 'g');
        end;
        function plotDipole(obj, cutoffRatio, regionAlpha, regionColor, createNewFigure, varargin)
            % plotDipole(obj, cutoffRatio, regionAlpha, regionColor, createNewFigure)
            
            if nargin < 2
                cutoffRatio = 0.9;
            end;
            
            if nargin < 3
                regionAlpha = 0.1;
            end;
            
            if nargin<4
                regionColor = 'g';
            end;
                        
            if nargin<5
                createNewFigure = true;
            end;
            
            if createNewFigure
                figure;
            end;
            
            dipoleId = obj.dipoleAndMeasure.getDipoleDensityContributionToRegionOfInterest(obj.membershipCube, obj.headGrid, cutoffRatio);
            plot_dipplot_with_cortex(obj.dipoleAndMeasure.location(dipoleId,:), true, 'coordformat', 'MNI', 'gui', 'off', 'spheres', 'on', varargin{:});
            
            plot_head_region(obj.headGrid, obj.membershipCube, regionColor, {'facealpha', regionAlpha, 'edgealpha', regionAlpha});
        end;
        function plotScalpMap(obj, varargin)
            dipoleId = obj.dipoleAndMeasure.getDipoleDensityContributionToRegionOfInterest(obj.membershipCube, obj.headGrid, varargin{:});
            obj.dipoleAndMeasure.scalpmap.plot(dipoleId, obj.scalpMapVisualizationPolarity);
        end;
        function plotMeanScalpMap(obj, varargin)
            dipoleId = obj.dipoleAndMeasure.getDipoleDensityContributionToRegionOfInterest(obj.membershipCube, obj.headGrid, varargin{:});
            meanScalpMap = obj.scalpMapVisualizationPolarity * squeeze(mean(obj.dipoleAndMeasure.scalpmap.normalizedChannelWeight(dipoleId,:,:), 1));
            
            figure;
            toporeplot(meanScalpMap, 'plotrad',0.5,  'intrad' , 0.5);
        end;
    end;
end